import React, { useState, useRef, useEffect } from 'react'
import { TextField, Button, Flex, Text } from '@radix-ui/themes'
import { Pencil1Icon } from '@radix-ui/react-icons'

interface EditableTitleProps {
  title: string
  onSave: (newTitle: string) => Promise<void>
  className?: string
}

export const EditableTitle: React.FC<EditableTitleProps> = ({
  title,
  onSave,
  className = ''
}) => {
  const [isEditing, setIsEditing] = useState(false)
  const [editValue, setEditValue] = useState(title)
  const [isSaving, setIsSaving] = useState(false)
  const inputRef = useRef<HTMLInputElement>(null)

  useEffect(() => {
    setEditValue(title)
  }, [title])

  useEffect(() => {
    if (isEditing && inputRef.current) {
      inputRef.current.focus()
      inputRef.current.select()
    }
  }, [isEditing])

  const handleStartEdit = () => {
    setIsEditing(true)
    setEditValue(title)
  }

  const handleSave = async () => {
    const trimmedValue = editValue.trim()
    if (trimmedValue === title || trimmedValue === '') {
      setIsEditing(false)
      return
    }

    setIsSaving(true)
    try {
      await onSave(trimmedValue)
      setIsEditing(false)
    } catch (error) {
      console.error('Failed to save title:', error)
      // Reset to original value on error
      setEditValue(title)
    } finally {
      setIsSaving(false)
    }
  }

  const handleCancel = () => {
    setIsEditing(false)
    setEditValue(title)
  }

  const handleKeyDown = (e: React.KeyboardEvent) => {
    if (e.key === 'Enter') {
      e.preventDefault()
      handleSave()
    } else if (e.key === 'Escape') {
      e.preventDefault()
      handleCancel()
    }
  }

  if (isEditing) {
    return (
      <Flex align="center" gap="2" className={className}>
        <TextField.Root
          ref={inputRef}
          size="2"
          value={editValue}
          onChange={(e) => setEditValue(e.target.value)}
          onKeyDown={handleKeyDown}
          onBlur={handleSave}
          disabled={isSaving}
          style={{ fontWeight: 'bold' }}
          maxLength={100}
        />
        {isSaving && (
          <Text size="2" color="gray">
            Saving...
          </Text>
        )}
      </Flex>
    )
  }

  return (
    <Button
      onClick={handleStartEdit}
      variant="ghost"
      size="2"
      className={`group ${className}`}
      title="Click to edit title"
    >
      <Text weight="bold" size="3">
        {title}
      </Text>
      <Pencil1Icon className="opacity-0 group-hover:opacity-100 transition-opacity duration-200" />
    </Button>
  )
} 